(ns clojure-magick.magick)
(import [org.im4java.core ConvertCmd IMOperation Stream2BufferedImage] )

(defn read-image "Read the image in the specified file; returns a BufferedImage." [filename]
  (let [op (doto (IMOperation.)
	     (.addImage (into-array String [filename]))
	     (.addImage (into-array String ["-"])))
	s2b (Stream2BufferedImage.)]
    (doto (ConvertCmd.)
      (.setOutputConsumer s2b)
      (.run op (into-array Object [])))
    (.getImage s2b))
)

(defn write-image "Write the image to the specified file." [img filename]
  (let [op (doto (IMOperation.)
	     (.addImage)
	     (.addImage))]
    (.run (ConvertCmd.) op  (into-array Object [img filename])))
)


(defn glue-vertical "Concatenate images in a sequence from top to bottom, producing a new image." [image & imgs]
  (let [imgs (cons image imgs)
	op (doto (IMOperation.)
	     (.addImage (count imgs))
	     (.append)
	     (.addImage (into-array String ["-"])))
	s2b (Stream2BufferedImage.)]
    (doto (ConvertCmd.)
      (.setOutputConsumer s2b)
      (.run op (into-array Object (vec imgs))))
    (.getImage s2b)
))

(defn glue-horizontal "Concatenate images in a sequence from left to right, producing a new image." [image & imgs]
  (let [imgs (cons image imgs)
	op (doto (IMOperation.)
	     (.addImage (count imgs))
	     (.p_append)
	     (.addImage (into-array String ["-"])))
	s2b (Stream2BufferedImage.)]
    (doto (ConvertCmd.)
      (.setOutputConsumer s2b)
      (.run op (into-array Object (vec imgs))))
    (.getImage s2b)
  ))



(defn resize "Resize the image, keeping the same aspect ratio."
  ([img width height]
  (let [op (doto (IMOperation.)
	     (.addImage)
	     (.resize width height)
	     (.addImage (into-array String ["-"])))
	s2b (Stream2BufferedImage.)]
    (doto (ConvertCmd.)
      (.setOutputConsumer s2b)
      (.run op (into-array Object [img])))
    (.getImage s2b)
    ))
  (
  [img width]
    (resize img width nil)
  )
)


(defn upside-down "Flip the image upside down."
  [img]
  (let [op (doto (IMOperation.)
	     (.addImage)
	     (.flip)
	     (.addImage (into-array String ["-"])))
	s2b (Stream2BufferedImage.)]
    (doto (ConvertCmd.)
      (.setOutputConsumer s2b)
      (.run op (into-array Object [img])))
    (.getImage s2b)
    )
)


(defn scale-rotate-translate "Scale the image, then rotate the image about the axis coordinates, then translate the image by the given amount. Does not change the dimensions of the picture."
  ([img axis-x axis-y scale-x scale-y degrees translate-x translate-y]
     (let [arg-str (apply str (list axis-x "," axis-y " " scale-x "," scale-y " " degrees " " translate-x "," translate-y))
	   op (doto (IMOperation.)
		(.addImage)
		(.distort "ScaleRotateTranslate" arg-str)
		(.addImage (into-array String ["-"])))
	   s2b (Stream2BufferedImage.)]
       (doto (ConvertCmd.)
	 (.setOutputConsumer s2b)
	 (.run op (into-array Object [img])))
       (.getImage s2b)))
  ([img axis-x axis-y scale degrees translate-x translate-y]
     (recur axis-x axis-y scale scale degrees translate-x translate-y))
)
     

(defn rotate "Rotate the image clockwise by the given amount." [img degrees]
  (let [op (doto (IMOperation.)
	     (.addImage)
	     (.distort "ScaleRotateTranslate" (str degrees))
	     (.addImage (into-array String ["-"])))
	s2b (Stream2BufferedImage.)]
    (doto (ConvertCmd.)
      (.setOutputConsumer s2b)
      (.run op (into-array Object [img])))
    (.getImage s2b))
)


























(defn resize[]
  (let [op (doto (IMOperation.)
	     (.addImage (into-array String ["/home/r/jiggly.gif"]))
	     (.resize 800 600)
	     (.addImage (into-array String ["/home/r/jiggly_sm.gif"]))
	     )]
    (.run (ConvertCmd.) op (into-array Object []))))


(defn run-test[]
  (let [puff (read-image "/home/r/jiggly.gif")]
    (write-image (glue-horizontal puff puff puff) "/home/r/multijiggly.gif")
    ))
